local name, ns = ...
local debug = ns.debug
local Addon = _G[name]

local L = setmetatable({}, { __index = function(self, index) return index end })

local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")

local function escape(text)
	return text:replace("|","||")
end

local function unescape(text)
	return text:replace("||", "|")
end

local function setLineText(info, value)
	local side = info[#info]
	local num = info.arg
	Addon.db.profile[side][num] = unescape(value)
end

local function getLineText(info)
	local side = info[#info]
	local num = info.arg
	return escape(Addon.db.profile[side][num])
end

local function makeLineOption(options, num)
	options.args["line" .. num] = {
		type = 'group',
		name = L["Line %d"]:format(num),
		desc = L["Line"],
		inline = true,
		order = num + 100,
		args = {
			lefts = {
				type = 'input',
				name = L["left"],
				desc = L["left text"],
				multiline = true,
				width = "double",
				get = getLineText,
				set = setLineText,
				arg = num,
				order = 1,
			},
			rights = {
				type = 'input',
				name = L["right"],
				desc = L["right text"],
				multiline = true,
				width = "double",
				get = getLineText,
				set = setLineText,
				arg = num,
				order = 2,
			},
		},
	}
end

local function showConfig()
	AceConfigDialog:SetDefaultSize(name, 950, 600)
	AceConfigDialog:Open(name)
	-- InterfaceOptionsFrame:Hide() -- hides everything...
end

function setOption(info, value)
	local key = info[#info]
	Addon.db.profile[key] = value
end
function getOption(info)
	local key = info[#info]
	return Addon.db.profile[key]
end

local options
function Addon:CreateOptions()
	if options then
		return
	end
	options = {
		name = L["Tooltip LuaText"],
		desc = L[""],
		type = 'group',
		set = setOption,
		get = getOption,
		args = {
			general = {
				type = 'group',
				name = L["General"],
				desc = L["General"],
				order = 1,
				args = {
					alwaysShowLefts = {
						type = 'toggle',
						name = L["Always show texts"],
						desc = L["Always show left texts, even if the right text is empty"],
						order = 1,
					},
					handleErrors = {
						type = 'toggle',
						name = L["Handle text's lua-errors"],
						desc = L["Whether to handle lua-errors caused by custom texts. Handling the errors may take up more CPU."],
						order = 2,
						set = function(info, value)
								setOption(info, value)
								Addon:UpdateErrorHandler()
							end,
					},
					printErrors = {
						type = 'toggle',
						name = L["Show Errors"],
						desc = L["Print errors to Chatframe"],
						order = 3,
						disabled = function() return not Addon.db.profile.handleErrors end,
					},
				},
			},
			profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db),
			texts = {
				type = 'group',
				name = L["Texts"],
				desc = L["Edit Text-codes"],
				args = {},
				order = 2,
			}
		},
	}
	local dbp = self.db.profile
	for i = 1,30 do
		makeLineOption(options.args.texts, i)
	end

	AceConfig:RegisterOptionsTable(name, options)

	local options2 = {
		name = L["Tooltip LuaText"],
		desc = L[""],
		type = 'group',
		handler = self,
		args = {
			showConfig = {
				name = L["Open Configuration"],
				desc = L["Open standalone Configuration"],
				type = 'execute',
				func = "DelayedShowConfig",
			},
		},
	}
	
	AceConfig:RegisterOptionsTable("Tooltip_LuaText_bliz", options2)
	AceConfigDialog:AddToBlizOptions("Tooltip_LuaText_bliz", "Tooltip LuaText")
	
	self:RegisterChatCommand("ttl", showConfig)
	self:RegisterChatCommand("tooltiplua", showConfig)
end

function Addon:DelayedShowConfig()
	self:ScheduleTimer(showConfig, 0.1)
	InterfaceOptionsFrame:Hide()
end
